/**
IBM SOFTWARE DISCLAIMER 

Java array package (draft 0.2). Copyright (1998), International Business 
Machines Corporation.

Permission to use, copy, modify and distribute this software for any
noncommercial purpose and without fee is hereby granted, provided that
this copyright and permission notice appear on all copies of the
software. The name of the IBM Corporation may not be used in any
advertising or publicity pertaining to the use of the software. IBM
makes no warranty or representations about the suitability of the
software for any purpose.  It is provided "AS IS" without any express
or implied warranty, including the implied warranties of
merchantability, fitness for a particular purpose and non-infringement.
IBM shall not be liable for any direct, indirect, special or
consequential damages resulting from the loss of use, data or projects,
whether in an action of contract or tort, arising out of or in
connection with the use or performance of this software.
*/

import array.*;

public class Test {

    public static void testsection(int m, int n, int p, 
				   int s0, int s1, int s2,
				   doubleArray3D A) 
	throws InvalidArrayShapeException, InvalidRangeException {

	int i;
	for (i=0; i<m; i++) {
	    testsection(i,n,p,s0,s1,s2,
			A.section(i,new Range(0,n-1),new Range (0,p-1)));
	}
	int j;
	for (j=0; j<n; j++) {
	    testsection(j,m,p,s1,s0,s2,
			A.section(new Range(0,m-1),j,new Range (0,p-1)));
	}
	int k;
	for (k=0; k<p; k++) {
	    testsection(k,m,n,s2,s0,s1,
			A.section(new Range(0,m-1),new Range(0,n-1),k));
	}
    }

    public static void testsection(int i, int n, int p, 
				   int s0, int s1, int s2,
				   doubleArray2D A) 
	throws InvalidArrayShapeException, InvalidRangeException {

	int j;
	for (j=0; j<n; j++) {
	    testsection(i,j,p,s0,s1,s2,A.section(j,new Range(0,p-1)));
	}
	int k;
	for (k=0; k<p; k++) {
	    testsection(i,k,p,s0,s2,s1,A.section(new Range(0,n-1),k));
	}
    }

    public static void testsection(int i, int j, int p, 
				   int s0, int s1, int s2,
				   doubleArray1D A) 
	throws InvalidArrayShapeException {

	int k;
	for (k=0; k<p; k++) {
	    testsection(i,j,k,s0,s1,s2,A.section(k));
	}
    }

    public static void testsection(int i, int j, int k, 
				   int s0, int s1, int s2,
				   doubleArray0D A) {

	double element = i*s0+j*s1+k*s2;
	if (A.get() != element) {
	    System.out.println("Mismatch (i="+i+",j="+j+",k="+k+
			       ") "+A.get()+" != "+element);
	}
    }

    public static void main(String[] args) 
	throws InvalidArrayShapeException, InvalidRangeException {

	int m, n, p;
	m = n = p = 64;

	// create a 3D array
	doubleArray3D A = new doubleArray3D(m,n,p);

	int i,j,k;

	// initialize each of its elements
	for (i=0; i<m; i++) {
	    for (j=0; j<n; j++) {
		for (k=0; k<p; k++) {
		    A.set(i,j,k,i*1000000+j*1000+k);
		}
	    }
	}

	// test that the elements were written appropriately
	for (i=0; i<m; i++) {
	    for (j=0; j<n; j++) {
		for (k=0; k<p; k++) {
		    double element = A.get(i,j,k);
		    if (element != i*1000000+j*1000+k) {
			System.out.println("Mismatch (i="+i+",j="+j+
					   ",k="+k+") "+A.get(i,j,k)+
					   " != "+element);
		    }
		}
	    }
	}

	// test that the elements were written correclty
	testsection(m,n,p,1000000,1000,1,A);

	// now test sectioning operations for 0D arrays
	for (i=0; i<m; i++) {
	    for (j=0; j<n; j++) {
		for (k=0; k<p; k++) {
		    testsection(i,j,k,1000000,1000,1,A.section(i,j,k));
		}
	    }
	}

	// now test sectioning operations for 1D arrays
	for (i=0; i<m; i++) {
	    for (j=0; j<n; j++) {
		testsection(i,j,p,1000000,1000,1,
			    A.section(i,j,new Range(0,p-1)));
	    }
	}
	for (i=0; i<m; i++) {
	    for (k=0; k<p; k++) {
		testsection(i,k,p,1000000,1,1000,
			    A.section(i,new Range(0,n-1),k));
	    }
	}
	for (j=0; j<n; j++) {
	    for (k=0; k<p; k++) {
		testsection(j,k,m,1000,1,1000000,
			    A.section(new Range(0,m-1),j,k));
	    }
	}

	// now test sectioning operations for 2D arrays
	for (j=0; j<n; j++) {
	    testsection(j,m,p,1000,1000000,1,
			A.section(new Range(0,m-1),j, new Range(0,p-1)));
	}
	for (i=0; i<m; i++) {
	    testsection(i,n,p,1000000,1000,1,
			A.section(i,new Range(0,n-1), new Range(0,p-1)));
	}
	for (k=0; k<p; k++) {
	    testsection(k,m,n,1,1000000,1000,
			A.section(new Range(0,m-1),new Range(0,n-1), k));
	}

	// now test sectioning operations for 3D arrays
	testsection(m,n,p,1000000,1000,1,
		    A.section(new Range(0,m-1),new Range(0,n-1),new Range(0,p-1)));
    }
}

	
